<?php


namespace SwaggerBake\Test\TestCase\Lib\Annotations;

use Cake\Routing\Router;
use Cake\Routing\RouteBuilder;
use Cake\TestSuite\TestCase;
use SwaggerBake\Lib\AnnotationLoader;
use SwaggerBake\Lib\CakeModel;
use SwaggerBake\Lib\CakeRoute;
use SwaggerBake\Lib\Configuration;
use SwaggerBake\Lib\Swagger;

class SwagResponseSchemaTest extends TestCase
{
    public $fixtures = [
        'plugin.SwaggerBake.Employees',
    ];

    private $router;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $router = new Router();
        $router::scope('/api', function (RouteBuilder $builder) {
            $builder->setExtensions(['json']);
            $builder->resources('Employees', [
                'map' => [
                    'customResponseSchema' => [
                        'action' => 'customResponseSchema',
                        'method' => 'GET',
                        'path' => 'custom-response-schema'
                    ],
                ]
            ]);
        });
        $this->router = $router;

        $this->config = new Configuration([
            'prefix' => '/api',
            'yml' => '/config/swagger-with-existing.yml',
            'json' => '/webroot/swagger.json',
            'webPath' => '/swagger.json',
            'hotReload' => false,
            'exceptionSchema' => 'Exception',
            'requestAccepts' => ['application/x-www-form-urlencoded'],
            'responseContentTypes' => ['application/json'],
            'namespaces' => [
                'controllers' => ['\SwaggerBakeTest\App\\'],
                'entities' => ['\SwaggerBakeTest\App\\'],
                'tables' => ['\SwaggerBakeTest\App\\'],
            ]
        ], SWAGGER_BAKE_TEST_APP);

        AnnotationLoader::load();
    }

    public function testSwagResponseSchema()
    {
        $cakeRoute = new CakeRoute($this->router, $this->config);

        $swagger = new Swagger(new CakeModel($cakeRoute, $this->config));
        $arr = json_decode($swagger->toString(), true);

        $operation = $arr['paths']['/employees/custom-response-schema']['get'];

        $this->assertArrayHasKey('$ref', $operation['responses']['200']['content']['application/json']['schema']);
        $schema = $operation['responses']['200']['content']['application/json']['schema'];
        $this->assertEquals('#/components/schemas/Pet', $schema['$ref']);

        $this->assertArrayHasKey('400', $operation['responses']);
        $this->assertEquals('deprecated httpCode still works', $operation['responses']['400']['description']);

        $this->assertArrayHasKey('404', $operation['responses']);
        $this->assertEquals('new statusCode', $operation['responses']['404']['description']);

        $this->assertArrayHasKey('5XX', $operation['responses']);
        $this->assertEquals('status code range', $operation['responses']['5XX']['description']);

    }
}