<?php

namespace SwaggerBake\Test\TestCase\Lib\Operation;

use Cake\Controller\Controller;
use Cake\Routing\Route\Route;
use Cake\Routing\RouteBuilder;
use Cake\Routing\Router;
use Cake\TestSuite\TestCase;
use SwaggerBake\Lib\Annotation\SwagSecurity;
use SwaggerBake\Lib\CakeRoute;
use SwaggerBake\Lib\Configuration;
use SwaggerBake\Lib\Decorator\RouteDecorator;
use SwaggerBake\Lib\Factory\SwaggerFactory;
use SwaggerBake\Lib\OpenApi\Operation;
use SwaggerBake\Lib\Operation\OperationSecurity;

class OperationSecurityTest extends TestCase
{
    /** @var OperationSecurity  */
    private $operationSecurity;

    /** @var \SwaggerBake\Lib\Swagger  */
    private $swagger;

    /** @var RouteDecorator  */
    private $routeDecorator;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $router = new Router();
        $router::scope('/api', function (RouteBuilder $builder) {
            $builder->setExtensions(['json']);
            $builder->resources('Employees', [
                'only' => 'index'
            ]);
        });

        $this->config = [
            'prefix' => '/api',
            'yml' => '/config/swagger-bare-bones.yml',
            'json' => '/webroot/swagger.json',
            'webPath' => '/swagger.json',
            'hotReload' => false,
            'exceptionSchema' => 'Exception',
            'requestAccepts' => ['application/x-www-form-urlencoded'],
            'responseContentTypes' => ['application/json'],
            'namespaces' => [
                'controllers' => ['\SwaggerBakeTest\App\\'],
                'entities' => ['\SwaggerBakeTest\App\\'],
                'tables' => ['\SwaggerBakeTest\App\\'],
            ]
        ];

        $this->routeDecorator = new RouteDecorator(
            new Route('/api/employees/:id', [
                '_method' => ['GET'],
                'plugin' => '',
                'controller' => 'Employees',
                'action' => 'view'
            ])
        );

        $configuration = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);
        $cakeRoute = new CakeRoute($router, $configuration);
        $this->swagger = (new SwaggerFactory($configuration, $cakeRoute))->create();
    }

    public function testGetOperationSecurityFromSwagSecurityAnnotation()
    {
        $operationSecurity = new OperationSecurity(
            new Operation(),
            [new SwagSecurity(['name' => 'BearerAuth' , 'scopes' => ['read','write']])],
            $this->routeDecorator,
            new Controller(),
            $this->swagger
        );
        $operation = $operationSecurity->getOperationWithSecurity();
        $securities = $operation->getSecurity();
        $security = reset($securities);
        $this->assertEquals('BearerAuth', $security->getName());
        $this->assertCount(2, $security->getScopes());
    }

    public function testGetOperationSecurityFromAuthenticationComponent()
    {
        $controller = new Controller();
        $controller->loadComponent('Authentication.Authentication');

        $operationSecurity = new OperationSecurity(
            new Operation(),
            [],
            $this->routeDecorator,
            $controller,
            $this->swagger
        );
        $operation = $operationSecurity->getOperationWithSecurity();
        $securities = $operation->getSecurity();
        $security = reset($securities);
        $this->assertEquals('BearerAuth', $security->getName());
        $this->assertCount(0, $security->getScopes());
    }
}